@@ -36,7 +36,10 @@ G_DEFINE_TYPE (TrackerTagBar, tracker_tag_bar, GTK_TYPE_HBOX);
 
 /* FORWARD DECLARATIONS */
 
+#if 0
 static void _tag_bar_add_tag (TrackerTagBar *bar, GtkWidget *box, const char *tag);
+#endif
+
 static void _tag_launch_search (const gchar *tag);
 
 /* STRUCTS & ENUMS */
@@ -49,7 +52,9 @@ struct _TrackerTagBarPrivate
 	gchar *uri;
 	const gchar *active_tag;
 
+#if 0
 	ServiceType type;
+#endif
 
 	GtkWidget *tag_box;
 	GtkWidget *add_button;
@@ -59,6 +64,8 @@ struct _TrackerTagBarPrivate
 	GtkWidget *entry;
 };
 
+#if 0
+
 /* CALLBACKS */
 static void
 _keywords_reply (char **array, GError *error, TrackerTagBar *bar)
@@ -131,6 +138,8 @@ _on_tag_button_press_event (GtkWidget			*button,
 	return FALSE;
 }
 
+#endif
+
 static void
 _tag_launch_search (const gchar *tag)
 {
@@ -164,22 +173,28 @@ static void
 remove_tag_activate_cb(GtkMenuItem *menu_item, TrackerTagBar *bar)
 {
 	TrackerTagBarPrivate *priv;
+#if 0
 	GError *error = NULL;
+#endif
 	char *args[1];
 
 	priv = TRACKER_TAG_BAR_GET_PRIVATE (bar);
 
 	args[0] = g_strdup (priv->active_tag);
 
+	/* TODO: Port to SPARQL */
+#if 0
 	tracker_keywords_remove(priv->client, priv->type, priv->uri,
 				 args, &error);
 	if (error) {
 		g_print ("Tag Removal Error : %s", error->message);
 		return;
 	}
+
 	gchar *temp = g_strdup (priv->uri);
 	tracker_tag_bar_set_uri (bar, priv->type, temp);
 	g_free (temp);
+#endif
 }
 
 static void
@@ -200,13 +215,17 @@ _on_apply_add_tag (GtkButton *but, TrackerTagBar *bar)
 {
 	TrackerTagBarPrivate *priv;
 	const gchar *text;
+#if 0
 	gchar **tags;
 	GError *error = NULL;
+#endif
 
 	priv = TRACKER_TAG_BAR_GET_PRIVATE (bar);
 
 	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
 
+	/* TODO: Port to SPARQL */
+#if 0
 	if (strcmp (text, "Type tags you want to add here, separated by commas") != 0) {
 
 		tags = g_strsplit (text, ",", 0);
@@ -223,6 +242,7 @@ _on_apply_add_tag (GtkButton *but, TrackerTagBar *bar)
 	gchar *temp = g_strdup (priv->uri);
 	tracker_tag_bar_set_uri (bar, priv->type, temp);
 	g_free (temp);
+#endif
 }
 
 static void
@@ -284,6 +304,8 @@ _on_add_tag_clicked (GtkButton *but, TrackerTagBar *bar)
 
 /* UTILS */
 
+#if 0
+
 static void
 _tag_bar_add_tag (TrackerTagBar *bar, GtkWidget *box, const char *tag)
 {
@@ -314,6 +336,7 @@ _tag_bar_add_tag (TrackerTagBar *bar, GtkWidget *box, const char *tag)
 }
 
 /* HEADER FUNCTIONS */
+	/* TODO: Port to SPARQL */
 void
 tracker_tag_bar_set_uri (TrackerTagBar *bar, ServiceType type, const gchar *uri)
 {
@@ -331,6 +354,8 @@ tracker_tag_bar_set_uri (TrackerTagBar *bar, ServiceType type, const gchar *uri)
 				    bar);
 }
 
+#endif
+
 /* TRACKER TAG BAR NEW */
 static void
 tracker_tag_bar_class_init (TrackerTagBarClass *class)
@@ -426,7 +451,7 @@ tracker_tag_bar_new (void)
 				NULL);
 	priv = TRACKER_TAG_BAR_GET_PRIVATE (tag_bar);
 
-	client = tracker_connect (TRUE);
+	client = tracker_connect (TRUE, -1);
 	priv->client = client;
 	return tag_bar;
 }
