@@ -20,7 +20,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
- 
+
 #ifndef TRACKER_UI_H
 #define TRACKER_UI_H
 
@@ -28,18 +28,18 @@
 #include <gtk/gtk.h>
 
 /**
- * The GtkTargetEntry to use as the drag type when 
+ * The GtkTargetEntry to use as the drag type when
  * dragging and dropping keywords
  **/
 const GtkTargetEntry KEYWORD_DRAG_TYPES[] = {
-	{"property/keyword", 0, 0 }
+	{ (gchar *) "property/keyword", 0, 0 }
 };
 
 void
-tracker_render_emblem_pixbuf_cb (GtkCellLayout   *cell_layout,
-                            	 GtkCellRenderer *cell,
-                            	 GtkTreeModel    *tree_model,
-                            	 GtkTreeIter     *iter,
-                            	 gpointer        user_data);
+tracker_render_emblem_pixbuf_cb (GtkCellLayout	 *cell_layout,
+				 GtkCellRenderer *cell,
+				 GtkTreeModel	 *tree_model,
+				 GtkTreeIter	 *iter,
+				 gpointer	 user_data);
 
 #endif /* TRACKER_UI_H */
