@@ -21,6 +21,7 @@
 #include "config.h"
 
 #include <libtracker-common/tracker-dbus.h>
+#include <libtracker-common/tracker-file-utils.h>
 
 #include "tracker-crawler.h"
 #include "tracker-marshal.h"
@@ -57,6 +58,19 @@ typedef struct {
 	TrackerSparqlBuilder *builder;
 } ProcessData;
 
+typedef struct {
+	GMainLoop *main_loop;
+	gboolean value;
+} SparqlQueryData;
+
+typedef struct {
+	GMainLoop *main_loop;
+	gint       level;
+	GString   *sparql;
+	const gchar *source_uri;
+	const gchar *uri;
+} RecursiveMoveData;
+
 struct TrackerMinerFSPrivate {
 	TrackerMonitor *monitor;
 	TrackerCrawler *crawler;
@@ -66,6 +80,7 @@ struct TrackerMinerFSPrivate {
 	GQueue         *items_updated;
 	GQueue         *items_deleted;
 	GQueue         *items_moved;
+	GHashTable     *items_writeback;
 
 	GQuark          quark_ignore_file;
 
@@ -106,7 +121,8 @@ enum {
 	QUEUE_CREATED,
 	QUEUE_UPDATED,
 	QUEUE_DELETED,
-	QUEUE_MOVED
+	QUEUE_MOVED,
+	QUEUE_WRITEBACK
 };
 
 enum {
@@ -115,6 +131,7 @@ enum {
 	CHECK_DIRECTORY_CONTENTS,
 	MONITOR_DIRECTORY,
 	PROCESS_FILE,
+	WRITEBACK_FILE,
 	FINISHED,
 	LAST_SIGNAL
 };
@@ -144,6 +161,8 @@ static void           miner_started                (TrackerMiner   *miner);
 static void           miner_stopped                (TrackerMiner   *miner);
 static void           miner_paused                 (TrackerMiner   *miner);
 static void           miner_resumed                (TrackerMiner   *miner);
+static void           miner_writeback              (TrackerMiner   *miner,
+                                                    const GStrv     subjects);
 
 static DirectoryData *directory_data_new           (GFile          *file,
 						    gboolean        recurse);
@@ -192,6 +211,10 @@ static void           crawl_directories_stop       (TrackerMinerFS *fs);
 
 static void           item_queue_handlers_set_up   (TrackerMinerFS *fs);
 
+static void           item_update_uri_recursively (TrackerMinerFS    *fs,
+						   RecursiveMoveData *data,
+						   const gchar       *source_uri,
+						   const gchar       *uri);
 
 static guint signals[LAST_SIGNAL] = { 0, };
 
@@ -212,6 +235,7 @@ tracker_miner_fs_class_init (TrackerMinerFSClass *klass)
         miner_class->stopped = miner_stopped;
 	miner_class->paused  = miner_paused;
 	miner_class->resumed = miner_resumed;
+	miner_class->writeback = miner_writeback;
 
 	fs_class->check_file        = fs_defaults;
 	fs_class->check_directory   = fs_defaults;
@@ -347,6 +371,33 @@ tracker_miner_fs_class_init (TrackerMinerFSClass *klass)
 			      tracker_marshal_BOOLEAN__OBJECT_OBJECT_OBJECT,
 			      G_TYPE_BOOLEAN,
 			      3, G_TYPE_FILE, TRACKER_TYPE_SPARQL_BUILDER, G_TYPE_CANCELLABLE),
+
+	/**
+	 * TrackerMinerFS::writeback-file:
+	 * @miner_fs: the #TrackerMinerFS
+	 * @file: a #GFile
+	 * @builder: a #TrackerSparqlBuilder
+	 * @cancellable: a #GCancellable
+	 *
+	 * The ::writeback-file signal is emitted whenever a file should
+	 * be marked as writeback, and it's metadata prepared for that.
+	 *
+	 * @builder is the #TrackerSparqlBuilder where all sparql updates
+	 * to be performed for @file will be appended.
+	 *
+	 * Returns: %TRUE on success
+	 *          %FALSE on failure
+	 **/
+	signals[WRITEBACK_FILE] =
+		g_signal_new ("writeback-file",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (TrackerMinerFSClass, writeback_file),
+			      NULL, NULL,
+			      tracker_marshal_BOOLEAN__OBJECT_OBJECT_OBJECT,
+			      G_TYPE_BOOLEAN,
+			      3, G_TYPE_FILE, TRACKER_TYPE_SPARQL_BUILDER, G_TYPE_CANCELLABLE),
+
 	/**
 	 * TrackerMinerFS::finished:
 	 * @miner_fs: the #TrackerMinerFS
@@ -393,6 +444,9 @@ tracker_miner_fs_init (TrackerMinerFS *object)
 	priv->items_updated = g_queue_new ();
 	priv->items_deleted = g_queue_new ();
 	priv->items_moved = g_queue_new ();
+	priv->items_writeback = g_hash_table_new_full (g_str_hash, g_str_equal,
+	                                               (GDestroyNotify) g_free,
+	                                               (GDestroyNotify) NULL);
 
 	/* Set up the crawlers now we have config and hal */
 	priv->crawler = tracker_crawler_new ();
@@ -436,10 +490,16 @@ process_data_new (GFile                *file,
 {
 	ProcessData *data;
 
-	data = g_slice_new (ProcessData);
+	data = g_slice_new0 (ProcessData);
 	data->file = g_object_ref (file);
-	data->cancellable = g_object_ref (cancellable);
-	data->builder = g_object_ref (builder);
+
+	if (cancellable) {
+		data->cancellable = g_object_ref (cancellable);
+	}
+
+	if (builder) {
+		data->builder = g_object_ref (builder);
+	}
 
 	return data;
 }
@@ -448,8 +508,15 @@ static void
 process_data_free (ProcessData *data)
 {
 	g_object_unref (data->file);
-	g_object_unref (data->cancellable);
-	g_object_unref (data->builder);
+
+	if (data->cancellable) {
+		g_object_unref (data->cancellable);
+	}
+
+	if (data->builder) {
+		g_object_unref (data->builder);
+	}
+
 	g_slice_free (ProcessData, data);
 }
 
@@ -511,6 +578,8 @@ fs_finalize (GObject *object)
 	g_queue_foreach (priv->items_created, (GFunc) g_object_unref, NULL);
 	g_queue_free (priv->items_created);
 
+	g_hash_table_unref (priv->items_writeback);
+
 	G_OBJECT_CLASS (tracker_miner_fs_parent_class)->finalize (object);
 }
 
@@ -560,7 +629,7 @@ fs_get_property (GObject    *object,
 	}
 }
 
-static gboolean 
+static gboolean
 fs_defaults (TrackerMinerFS *fs,
 	     GFile          *file)
 {
@@ -584,8 +653,8 @@ miner_started (TrackerMiner *miner)
 
 	fs->private->been_started = TRUE;
 
-	g_object_set (miner, 
-		      "progress", 0.0, 
+	g_object_set (miner,
+		      "progress", 0.0,
 		      "status", _("Initializing"),
 		      NULL);
 
@@ -595,8 +664,8 @@ miner_started (TrackerMiner *miner)
 static void
 miner_stopped (TrackerMiner *miner)
 {
-	g_object_set (miner, 
-		      "progress", 1.0, 
+	g_object_set (miner,
+		      "progress", 1.0,
 		      "status", _("Idle"),
 		      NULL);
 }
@@ -640,6 +709,25 @@ miner_resumed (TrackerMiner *miner)
 	}
 }
 
+
+static void
+miner_writeback (TrackerMiner *miner, const GStrv subjects)
+{
+	TrackerMinerFS *fs;
+	guint n;
+
+	fs = TRACKER_MINER_FS (miner);
+
+	for (n = 0; subjects[n] != NULL; n++) {
+		g_hash_table_insert (fs->private->items_writeback,
+		                     g_strdup (subjects[n]),
+		                     GINT_TO_POINTER (TRUE));
+	}
+
+	item_queue_handlers_set_up (fs);
+}
+
+
 static DirectoryData *
 directory_data_new (GFile    *file,
 		    gboolean  recurse)
@@ -689,17 +777,32 @@ process_print_stats (TrackerMinerFS *fs)
 }
 
 static void
-process_stop (TrackerMinerFS *fs) 
+commit_cb (GObject      *object,
+           GAsyncResult *result,
+           gpointer      user_data)
+{
+	TrackerMiner *miner = TRACKER_MINER (object);
+	GError *error = NULL;
+
+	tracker_miner_commit_finish (miner, result, &error);
+
+	if (error) {
+		g_critical ("Could not commit: %s", error->message);
+	}
+}
+
+static void
+process_stop (TrackerMinerFS *fs)
 {
 	/* Now we have finished crawling, print stats and enable monitor events */
 	process_print_stats (fs);
 
-	tracker_miner_commit (TRACKER_MINER (fs));
+	tracker_miner_commit (TRACKER_MINER (fs), NULL, commit_cb, NULL);
 
 	g_message ("Idle");
 
-	g_object_set (fs, 
-		      "progress", 1.0, 
+	g_object_set (fs,
+		      "progress", 1.0,
 		      "status", _("Idle"),
 		      NULL);
 
@@ -744,6 +847,56 @@ item_moved_data_free (ItemMovedData *data)
 	g_slice_free (ItemMovedData, data);
 }
 
+static void
+sparql_update_cb (GObject      *object,
+                  GAsyncResult *result,
+                  gpointer      user_data)
+{
+	TrackerMinerFS *fs;
+	TrackerMinerFSPrivate *priv;
+	ProcessData *data;
+
+	GError *error = NULL;
+
+	tracker_miner_execute_update_finish (TRACKER_MINER (object), result, &error);
+
+	fs = TRACKER_MINER_FS (object);
+	priv = fs->private;
+	data = user_data;
+
+	if (error) {
+		g_critical ("Could not execute sparql: %s", error->message);
+	} else {
+		if (fs->private->been_crawled) {
+			/* Only commit immediately for
+			 * changes after initial crawling.
+			 */
+			tracker_miner_commit (TRACKER_MINER (fs), NULL, commit_cb, NULL);
+		}
+	}
+
+	priv->processing_pool = g_list_remove (priv->processing_pool, data);
+	process_data_free (data);
+
+	item_queue_handlers_set_up (fs);
+}
+
+static void
+sparql_query_cb (GObject      *object,
+                 GAsyncResult *result,
+                 gpointer      user_data)
+{
+	SparqlQueryData *data = user_data;
+	TrackerMiner *miner = TRACKER_MINER (object);
+
+	GError *error = NULL;
+
+	const GPtrArray *query_results = tracker_miner_execute_sparql_finish (miner, result, &error);
+
+	data->value = query_results && query_results->len == 1;
+	g_main_loop_quit (data->main_loop);
+}
+
 static void
 item_add_or_update_cb (TrackerMinerFS *fs,
 		       ProcessData    *data,
@@ -754,7 +907,17 @@ item_add_or_update_cb (TrackerMinerFS *fs,
 	uri = g_file_get_uri (data->file);
 
 	if (error) {
-		g_message ("Could not process '%s': %s", uri, error->message);
+		if (error->code == G_IO_ERROR_NOT_FOUND) {
+			g_message ("Could not process '%s': %s", uri, error->message);
+		} else {
+			g_critical ("Could not process '%s': %s", uri, error->message);
+		}
+
+		fs->private->processing_pool =
+			g_list_remove (fs->private->processing_pool, data);
+		process_data_free (data);
+
+		item_queue_handlers_set_up (fs);
 	} else {
 		gchar *full_sparql;
 
@@ -763,24 +926,15 @@ item_add_or_update_cb (TrackerMinerFS *fs,
 		full_sparql = g_strdup_printf ("DROP GRAPH <%s> %s",
 					       uri, tracker_sparql_builder_get_result (data->builder));
 
-		tracker_miner_execute_batch_update (TRACKER_MINER (fs), full_sparql, NULL);
+		tracker_miner_execute_batch_update (TRACKER_MINER (fs),
+						    full_sparql,
+						    NULL,
+						    sparql_update_cb,
+						    data);
 		g_free (full_sparql);
-
-		if (fs->private->been_crawled) {
-			/* Only commit immediately for
-			 * changes after initial crawling.
-			 */
-			tracker_miner_commit (TRACKER_MINER (fs));
-		}
 	}
 
-	fs->private->processing_pool =
-		g_list_remove (fs->private->processing_pool, data);
-	process_data_free (data);
-
 	g_free (uri);
-
-	item_queue_handlers_set_up (fs);
 }
 
 static gboolean
@@ -849,29 +1003,39 @@ item_query_exists (TrackerMinerFS *miner,
 {
 	gboolean   result;
 	gchar     *sparql, *uri;
-	GPtrArray *sparql_result;
+	SparqlQueryData data;
 
 	uri = g_file_get_uri (file);
 	sparql = g_strdup_printf ("SELECT ?s WHERE { ?s a rdfs:Resource . FILTER (?s = <%s>) }",
 	                          uri);
 
-	sparql_result = tracker_miner_execute_sparql (TRACKER_MINER (miner), sparql, NULL);
+	data.main_loop = g_main_loop_new (NULL, FALSE);
+	data.value = FALSE;
 
-	result = (sparql_result && sparql_result->len == 1);
+	tracker_miner_execute_sparql (TRACKER_MINER (miner),
+				      sparql,
+				      NULL,
+				      sparql_query_cb,
+				      &data);
+
+	g_main_loop_run (data.main_loop);
+	result = data.value;
+
+	g_main_loop_unref (data.main_loop);
 
-	tracker_dbus_results_ptr_array_free (&sparql_result);
 	g_free (sparql);
 	g_free (uri);
 
 	return result;
 }
 
-static void
+static gboolean
 item_remove (TrackerMinerFS *fs,
 	     GFile          *file)
 {
 	GString *sparql;
 	gchar *uri, *slash_uri;
+	ProcessData *data;
 
 	uri = g_file_get_uri (file);
 
@@ -880,7 +1044,7 @@ item_remove (TrackerMinerFS *fs,
 
 	if (!item_query_exists (fs, file)) {
 		g_debug ("  File does not exist anyway (uri:'%s')", uri);
-		return;
+		return TRUE;
 	}
 
 	if (!g_str_has_suffix (uri, "/")) {
@@ -893,66 +1057,191 @@ item_remove (TrackerMinerFS *fs,
 
 	/* Delete all children */
 	g_string_append_printf (sparql,
-				"DELETE { ?u a rdfs:Resource } "
+				"DELETE FROM <%s> { ?u a rdfs:Resource } "
 				"WHERE { ?u nfo:belongsToContainer ?p . FILTER (fn:starts-with (?p, \"%s\")) } ",
-				slash_uri);
+				uri, slash_uri);
 
 	/* Delete resource itself */
 	g_string_append_printf (sparql,
-				"DELETE { <%s> a rdfs:Resource }",
-				uri);
+				"DELETE FROM <%s> { <%s> a rdfs:Resource }",
+				uri, uri);
 
-	tracker_miner_execute_batch_update (TRACKER_MINER (fs), sparql->str, NULL);
+	data = process_data_new (file, NULL, NULL);
+	fs->private->processing_pool = g_list_prepend (fs->private->processing_pool, data);
+
+	tracker_miner_execute_batch_update (TRACKER_MINER (fs),
+					    sparql->str,
+					    NULL,
+					    sparql_update_cb,
+					    data);
 
 	g_string_free (sparql, TRUE);
 	g_free (slash_uri);
 	g_free (uri);
+
+	return FALSE;
+}
+
+static gboolean
+item_writeback (TrackerMinerFS *fs,
+                GFile          *file,
+                GFile          *source_file)
+{
+	TrackerSparqlBuilder *sparql;
+	gchar *uri;
+	gboolean success = FALSE;
+	GCancellable *cancellable;
+	GFile *working_file;
+
+	/* While we are in writeback:
+	 * o. We always ignore deletes because it's never the final operation
+	 *    of a writeback. We have a delete when both are null.
+	 * o. A create means the writeback used rename(). This is the final 
+	 *    operation of a writeback and thus we make the update query.
+	 *    We have a create when file == null and source_file != null
+	 * o. A move means the writeback used rename(). This is the final 
+	 *    operation of a writeback and thus we make the update query.
+	 *    We have a move when both file and source_file aren't null.
+	 * o. A update means the writeback didn't use rename(). This is the
+	 *    final operation of a writeback and thus we make the update query.
+	 *    An update means that file != null and source_file == null. */
+
+	/* Happens on delete while in writeback */
+	if (!file && !source_file) {
+		return TRUE;
+	}
+
+	/* Create or update, we are the final one so we make the update query */
+
+	if (!file && source_file) {
+		/* Happens on create while in writeback */
+		working_file = source_file;
+	} else {
+		/* Happens on update while in writeback */
+		working_file = file;
+	}
+
+	uri = g_file_get_uri (working_file);
+
+	g_debug ("Updating item: '%s' (Writeback event)", uri);
+
+	if (!item_query_exists (fs, working_file)) {
+		g_debug ("  File does not exist anyway (uri:'%s')", uri);
+		return TRUE;
+	}
+
+	cancellable = g_cancellable_new ();
+	sparql = tracker_sparql_builder_new_update ();
+	g_object_ref (working_file);
+
+	/* Writeback */
+	g_signal_emit (fs, signals[WRITEBACK_FILE], 0,
+	               working_file, sparql, cancellable, &success);
+
+	if (success) {
+		gchar *query;
+
+		/* Perhaps we should move the DELETE to tracker-miner-files.c? 
+		 * Or we add support for DELETE to TrackerSparqlBuilder ofcrs */
+
+		query = g_strdup_printf ("DELETE FROM <%s> { <%s> "
+		                           "nfo:fileSize ?unknown1 ;\n\t"
+		                           "nfo:fileLastModified ?unknown2 ;\n\t"
+		                           "nfo:fileLastAccessed ?unknown3 ;\n\t"
+		                           "nie:mimeType ?unknown4 \n"
+		                         "} WHERE { <%s> "
+		                           "nfo:fileSize ?unknown1 ;\n\t"
+		                           "nfo:fileLastModified ?unknown2 ;\n\t"
+		                           "nfo:fileLastAccessed ?unknown3 ;\n\t"
+		                           "nie:mimeType ?unknown4 \n"
+		                         "} \n %s", 
+		                         uri, uri, uri,
+		                         tracker_sparql_builder_get_result (sparql));
+
+		tracker_miner_execute_batch_update (TRACKER_MINER (fs),
+		                                    query,
+		                                    NULL, NULL, NULL);
+
+		g_free (query);
+	}
+
+	g_hash_table_remove (fs->private->items_writeback, uri);
+
+	g_object_unref (sparql);
+	g_object_unref (working_file);
+	g_object_unref (cancellable);
+
+	g_free (uri);
+
+	return FALSE;
 }
 
 static void
-item_update_uri_recursively (TrackerMinerFS *fs,
-			     GString        *sparql_update,
-			     const gchar    *source_uri,
-			     const gchar    *uri)
+item_update_uri_recursively_cb (GObject      *object,
+                                GAsyncResult *result,
+                                gpointer      user_data)
+{
+	TrackerMinerFS *fs = TRACKER_MINER_FS (object);
+	RecursiveMoveData *data = user_data;
+	GError *error = NULL;
+
+	const GPtrArray *query_results = tracker_miner_execute_sparql_finish (TRACKER_MINER (object), result, &error);
+
+	if (error) {
+		g_critical ("Could not query children: %s", error->message);
+	} else {
+		if (query_results) {
+			gint i;
+
+			for (i = 0; i < query_results->len; i++) {
+				gchar **child_source_uri, *child_uri;
+
+				child_source_uri = g_ptr_array_index (query_results, i);
+
+				if (!g_str_has_prefix (*child_source_uri, data->source_uri)) {
+					g_warning ("Child URI '%s' does not start with parent URI '%s'",
+						   *child_source_uri,
+						   data->source_uri);
+					continue;
+				}
+
+				child_uri = g_strdup_printf ("%s%s", data->uri, *child_source_uri + strlen (data->source_uri));
+
+				item_update_uri_recursively (fs, data, *child_source_uri, child_uri);
+
+				g_free (child_uri);
+			}
+		}
+	}
+
+	data->level--;
+
+	g_assert (data->level >= 0);
+
+	if (data->level == 0) {
+		g_main_loop_quit (data->main_loop);
+	}
+}
+
+static void
+item_update_uri_recursively (TrackerMinerFS    *fs,
+			     RecursiveMoveData *move_data,
+			     const gchar       *source_uri,
+			     const gchar       *uri)
 {
 	gchar *sparql;
-	GPtrArray *result_set;
 
-	g_debug ("Moving item from '%s' to '%s'",
-		 source_uri,
-		 uri);
+	move_data->level++;
 
-	g_string_append_printf (sparql_update, " <%s> tracker:uri <%s> .", source_uri, uri);
+	g_string_append_printf (move_data->sparql, " <%s> tracker:uri <%s> .", source_uri, uri);
 
 	sparql = g_strdup_printf ("SELECT ?child WHERE { ?child nfo:belongsToContainer <%s> }", source_uri);
-	result_set = tracker_miner_execute_sparql (TRACKER_MINER (fs), sparql, NULL);
+	tracker_miner_execute_sparql (TRACKER_MINER (fs),
+				      sparql,
+				      NULL,
+				      item_update_uri_recursively_cb,
+				      move_data);
 	g_free (sparql);
-
-	if (result_set) {
-		gint i;
-
-		for (i = 0; i < result_set->len; i++) {
-			gchar **child_source_uri, *child_uri;
-
-			child_source_uri = g_ptr_array_index (result_set, i);
-
-			if (!g_str_has_prefix (*child_source_uri, source_uri)) {
-				g_warning ("Child URI '%s' does not start with parent URI '%s'",
-				           *child_source_uri,
-				           source_uri);
-				continue;
-			}
-
-			child_uri = g_strdup_printf ("%s%s", uri, *child_source_uri + strlen (source_uri));
-
-			item_update_uri_recursively (fs, sparql_update, *child_source_uri, child_uri);
-
-			g_free (child_source_uri);
-			g_free (child_uri);
-		}
-
-		g_ptr_array_free (result_set, TRUE);
-	}
 }
 
 static gboolean
@@ -963,6 +1252,8 @@ item_move (TrackerMinerFS *fs,
 	gchar     *uri, *source_uri, *escaped_filename;
 	GFileInfo *file_info;
 	GString   *sparql;
+	RecursiveMoveData move_data;
+	ProcessData *data;
 
 	uri = g_file_get_uri (file);
 	source_uri = g_file_get_uri (source_file);
@@ -987,32 +1278,55 @@ item_move (TrackerMinerFS *fs,
 				       NULL, NULL);
 
 	if (!file_info) {
+		gboolean retval;
+
 		/* Destination file has gone away, ignore dest file and remove source if any */
-		item_remove (fs, source_file);
+		retval = item_remove (fs, source_file);
 
 		g_free (source_uri);
 		g_free (uri);
 
-		return TRUE;
+		return retval;
 	}
 
+	g_debug ("Moving item from '%s' to '%s'",
+		 source_uri,
+		 uri);
+
 	sparql = g_string_new ("");
 
 	g_string_append_printf (sparql,
-				"DELETE { <%s> nfo:fileName ?o } WHERE { <%s> nfo:fileName ?o }",
-				source_uri, source_uri);
+				"DELETE FROM <%s> { <%s> nfo:fileName ?o } WHERE { <%s> nfo:fileName ?o }",
+				source_uri, source_uri, source_uri);
 
-	g_string_append (sparql, " INSERT {");
+	g_string_append_printf (sparql, " INSERT INTO <%s> {", uri);
 
 	escaped_filename = g_strescape (g_file_info_get_display_name (file_info), NULL);
 
 	g_string_append_printf (sparql, " <%s> nfo:fileName \"%s\" .", source_uri, escaped_filename);
 
-	item_update_uri_recursively (fs, sparql, source_uri, uri);
+	move_data.main_loop = g_main_loop_new (NULL, FALSE);
+	move_data.level = 0;
+	move_data.sparql = sparql;
+	move_data.source_uri = source_uri;
+	move_data.uri = uri;
+
+	item_update_uri_recursively (fs, &move_data, source_uri, uri);
+
+	g_main_loop_run (move_data.main_loop);
+
+	g_main_loop_unref (move_data.main_loop);
 
 	g_string_append (sparql, " }");
 
-	tracker_miner_execute_batch_update (TRACKER_MINER (fs), sparql->str, NULL);
+	data = process_data_new (file, NULL, NULL);
+	fs->private->processing_pool = g_list_prepend (fs->private->processing_pool, data);
+
+	tracker_miner_execute_batch_update (TRACKER_MINER (fs),
+					    sparql->str,
+					    NULL,
+					    sparql_update_cb,
+					    data);
 
 	g_free (uri);
 	g_free (source_uri);
@@ -1022,6 +1336,18 @@ item_move (TrackerMinerFS *fs,
 	return TRUE;
 }
 
+static gboolean
+check_writeback (TrackerMinerFS *fs, GFile *queue_file)
+{
+	gchar *uri = g_file_get_uri (queue_file);
+	if (g_hash_table_lookup (fs->private->items_writeback, uri)) {
+		g_free (uri);
+		return TRUE;
+	}
+	g_free (uri);
+	return FALSE;
+}
+
 static gint
 item_queue_get_next_file (TrackerMinerFS  *fs,
 			  GFile          **file,
@@ -1033,14 +1359,23 @@ item_queue_get_next_file (TrackerMinerFS  *fs,
 	/* Deleted items first */
 	queue_file = g_queue_pop_head (fs->private->items_deleted);
 	if (queue_file) {
-		*file = queue_file;
 		*source_file = NULL;
+		if (check_writeback (fs, queue_file)) {
+			*file = NULL;
+			return QUEUE_WRITEBACK;
+		}
+		*file = queue_file;
 		return QUEUE_DELETED;
 	}
 
 	/* Created items next */
 	queue_file = g_queue_pop_head (fs->private->items_created);
 	if (queue_file) {
+		if (check_writeback (fs, queue_file)) {
+			*file = NULL;
+			*source_file = queue_file;
+			return QUEUE_WRITEBACK;
+		}
 		*file = queue_file;
 		*source_file = NULL;
 		return QUEUE_CREATED;
@@ -1051,6 +1386,8 @@ item_queue_get_next_file (TrackerMinerFS  *fs,
 	if (queue_file) {
 		*file = queue_file;
 		*source_file = NULL;
+		if (check_writeback (fs, queue_file))
+			return QUEUE_WRITEBACK;
 		return QUEUE_UPDATED;
 	}
 
@@ -1060,7 +1397,8 @@ item_queue_get_next_file (TrackerMinerFS  *fs,
 		*file = g_object_ref (data->file);
 		*source_file = g_object_ref (data->source_file);
 		item_moved_data_free (data);
-
+		if (check_writeback (fs, *file))
+			return QUEUE_WRITEBACK;
 		return QUEUE_MOVED;
 	}
 
@@ -1075,7 +1413,7 @@ item_queue_get_progress (TrackerMinerFS *fs)
 {
 	guint items_to_process = 0;
 	guint items_total = 0;
-	
+
 	items_to_process += g_queue_get_length (fs->private->items_deleted);
 	items_to_process += g_queue_get_length (fs->private->items_created);
 	items_to_process += g_queue_get_length (fs->private->items_updated);
@@ -1108,6 +1446,17 @@ item_queue_handlers_cb (gpointer user_data)
 	fs = user_data;
 	queue = item_queue_get_next_file (fs, &file, &source_file);
 
+	if (file && tracker_file_is_locked (file)) {
+		/* File is locked, ignore any updates on it */
+		g_object_unref (file);
+
+		if (source_file) {
+			g_object_unref (source_file);
+		}
+
+		return TRUE;
+	}
+
 	if (!fs->private->timer) {
 		fs->private->timer = g_timer_new ();
 	}
@@ -1136,12 +1485,15 @@ item_queue_handlers_cb (gpointer user_data)
 		keep_processing = item_move (fs, file, source_file);
 		break;
 	case QUEUE_DELETED:
-		item_remove (fs, file);
+		keep_processing = item_remove (fs, file);
 		break;
 	case QUEUE_CREATED:
 	case QUEUE_UPDATED:
 		keep_processing = item_add_or_update (fs, file);
 		break;
+	case QUEUE_WRITEBACK:
+		keep_processing = item_writeback (fs, file, source_file);
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1162,7 +1514,7 @@ item_queue_handlers_cb (gpointer user_data)
 			/* Only commit immediately for
 			 * changes after initial crawling.
 			 */
-			tracker_miner_commit (TRACKER_MINER (fs));
+			tracker_miner_commit (TRACKER_MINER (fs), NULL, commit_cb, NULL);
 		}
 
 		return TRUE;
@@ -1224,17 +1576,17 @@ should_change_index_for_file (TrackerMinerFS *fs,
 			      GFile          *file)
 {
 	gboolean            uptodate;
-	GPtrArray          *sparql_result;
 	GFileInfo          *file_info;
 	guint64             time;
 	time_t              mtime;
 	struct tm           t;
 	gchar              *query, *uri;
+	SparqlQueryData     data;
 
-	file_info = g_file_query_info (file, 
-				       G_FILE_ATTRIBUTE_TIME_MODIFIED, 
-				       G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, 
-				       NULL, 
+	file_info = g_file_query_info (file,
+				       G_FILE_ATTRIBUTE_TIME_MODIFIED,
+				       G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+				       NULL,
 				       NULL);
 	if (!file_info) {
 		/* NOTE: We return TRUE here because we want to update the DB
@@ -1259,12 +1611,20 @@ should_change_index_for_file (TrackerMinerFS *fs,
 				 t.tm_min,
 				 t.tm_sec,
 				 uri);
-	sparql_result = tracker_miner_execute_sparql (TRACKER_MINER (fs), query, NULL);
 
-	uptodate = sparql_result && sparql_result->len == 1;
+	data.main_loop = g_main_loop_new (NULL, FALSE);
+	data.value = FALSE;
 
-	tracker_dbus_results_ptr_array_free (&sparql_result);
+	tracker_miner_execute_sparql (TRACKER_MINER (fs),
+						      query,
+						      NULL,
+						      sparql_query_cb,
+						      &data);
 
+	g_main_loop_run (data.main_loop);
+	uptodate = data.value;
+
+	g_main_loop_unref (data.main_loop);
 	g_free (query);
 	g_free (uri);
 
@@ -1334,7 +1694,7 @@ monitor_item_created_cb (TrackerMonitor *monitor,
 		} else {
 			g_queue_push_tail (fs->private->items_created,
 					   g_object_ref (file));
-			
+
 			item_queue_handlers_set_up (fs);
 		}
 	}
@@ -1404,17 +1764,17 @@ monitor_item_deleted_cb (TrackerMonitor *monitor,
 	/* Remove directory from list of directories we are going to
 	 * iterate if it is in there.
 	 */
-	l = g_list_find_custom (fs->private->directories, 
-				path, 
+	l = g_list_find_custom (fs->private->directories,
+				path,
 				(GCompareFunc) g_strcmp0);
 
 	/* Make sure we don't remove the current device we are
 	 * processing, this is because we do this same clean up later
-	 * in process_device_next() 
+	 * in process_device_next()
 	 */
 	if (l && l != fs->private->current_directory) {
 		directory_data_free (l->data);
-		fs->private->directories = 
+		fs->private->directories =
 			g_list_delete_link (fs->private->directories, l);
 	}
 #endif
@@ -1440,7 +1800,7 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 
 			path = g_file_get_path (other_file);
 
-			g_debug ("Not in store:'?'->'%s' (DIR) (move monitor event, source unknown)", 
+			g_debug ("Not in store:'?'->'%s' (DIR) (move monitor event, source unknown)",
 				 path);
 
 			/* If the source is not monitored, we need to crawl it. */
@@ -1477,12 +1837,12 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 		} else if (!source_stored) {
 			/* Source file was not stored, check dest file as new */
 			if (!is_directory) {
-				g_queue_push_tail (fs->private->items_created, 
+				g_queue_push_tail (fs->private->items_created,
 						   g_object_ref (other_file));
-				
+
 				item_queue_handlers_set_up (fs);
 			} else {
-				g_debug ("Not in store:'?'->'%s' (DIR) (move monitor event, source monitored)", 
+				g_debug ("Not in store:'?'->'%s' (DIR) (move monitor event, source monitored)",
 					 path);
 
 				tracker_miner_fs_add_directory (fs, other_file, TRUE);
@@ -1490,7 +1850,7 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 		} else if (!should_process_other) {
 			/* Delete old file */
 			g_queue_push_tail (fs->private->items_deleted, g_object_ref (file));
-			
+
 			item_queue_handlers_set_up (fs);
 		} else {
 			/* Move old file to new file */
@@ -1499,7 +1859,7 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 
 			item_queue_handlers_set_up (fs);
 		}
-		
+
 		g_free (other_path);
 		g_free (path);
 	}
@@ -1540,7 +1900,7 @@ crawler_check_directory_cb (TrackerCrawler *crawler,
 	g_signal_emit (fs, signals[MONITOR_DIRECTORY], 0, file, &add_monitor);
 
 	/* FIXME: Should we add here or when we process the queue in
-	 * the finished sig? 
+	 * the finished sig?
 	 */
 	if (add_monitor) {
 		tracker_monitor_add (fs->private->monitor, file);
@@ -1552,7 +1912,7 @@ crawler_check_directory_cb (TrackerCrawler *crawler,
 	 * is not done recursively.
 	 *
 	 * As such, we only use the "check" rules here, we don't do
-	 * any database comparison with mtime. 
+	 * any database comparison with mtime.
 	 */
 	return should_check;
 }
