@@ -136,6 +136,10 @@ static gboolean miner_files_process_file      (TrackerMinerFS       *fs,
 					       GCancellable         *cancellable);
 static gboolean miner_files_monitor_directory (TrackerMinerFS       *fs,
 					       GFile                *file);
+static gboolean miner_files_writeback_file    (TrackerMinerFS       *fs,
+					       GFile                *file,
+					       TrackerSparqlBuilder *sparql,
+					       GCancellable         *cancellable);
 
 G_DEFINE_TYPE (TrackerMinerFiles, tracker_miner_files, TRACKER_TYPE_MINER_FS)
 
@@ -155,6 +159,7 @@ tracker_miner_files_class_init (TrackerMinerFilesClass *klass)
 	miner_fs_class->check_directory_contents = miner_files_check_directory_contents;
 	miner_fs_class->monitor_directory = miner_files_monitor_directory;
         miner_fs_class->process_file = miner_files_process_file;
+        miner_fs_class->writeback_file = miner_files_writeback_file;
 
        	g_object_class_install_property (object_class,
 					 PROP_CONFIG,
@@ -429,6 +434,26 @@ miner_files_constructed (GObject *object)
 	disk_space_check_start (mf);
 }
 
+static void
+set_up_mount_point_cb (GObject      *source,
+                       GAsyncResult *result,
+                       gpointer      user_data)
+{
+	gchar *removable_device_urn = user_data;
+
+	GError *error = NULL;
+	tracker_miner_execute_update_finish (TRACKER_MINER (source),
+	                                     result,
+	                                     &error);
+
+	if (error) {
+		g_critical ("Could not set up mount point '%s': %s",
+			    removable_device_urn, error->message);
+	}
+
+	g_free (removable_device_urn);
+}
+
 static void
 set_up_mount_point (TrackerMinerFiles *miner,
                     const gchar       *removable_device_urn,
@@ -437,7 +462,6 @@ set_up_mount_point (TrackerMinerFiles *miner,
                     GString           *accumulator)
 {
 	GString *queries;
-	GError *error = NULL;
 
 	g_debug ("Setting up mount point '%s'", removable_device_urn);
 
@@ -452,106 +476,118 @@ set_up_mount_point (TrackerMinerFiles *miner,
 			uri = g_file_get_uri (file);
 
 			g_string_append_printf (queries,
-			                        "DROP GRAPH <%s>\nINSERT { <%s> a tracker:Volume; tracker:mountPoint <%s> } ",
-			                        removable_device_urn, removable_device_urn, uri);
+			                        "DROP GRAPH <%s>\n "
+			                        "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:mountPoint <%s> } ",
+			                        removable_device_urn, removable_device_urn, removable_device_urn, uri);
 
 			g_object_unref (file);
 			g_free (uri);
 		}
 
 		g_string_append_printf (queries,
-		                        "DELETE { <%s> tracker:isMounted ?unknown } WHERE { <%s> a tracker:Volume; tracker:isMounted ?unknown } ",
+		                        "DELETE FROM <%s> { <%s> tracker:isMounted ?unknown } WHERE { <%s> a tracker:Volume; tracker:isMounted ?unknown } ",
+		                        removable_device_urn, removable_device_urn, removable_device_urn);
+
+		g_string_append_printf (queries,
+		                        "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:isMounted true } ",
 		                        removable_device_urn, removable_device_urn);
 
 		g_string_append_printf (queries,
-		                        "INSERT { <%s> a tracker:Volume; tracker:isMounted true } ",
-		                        removable_device_urn);
-
-		g_string_append_printf (queries,
-		                        "INSERT { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> } ",
-		                        removable_device_urn);
+		                        "INSERT INTO <%s> { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> } ",
+		                        removable_device_urn, removable_device_urn);
 	} else {
 		gchar *now;
 
 		now = tracker_date_to_string (time (NULL));
 
 		g_string_append_printf (queries,
-		                        "DELETE { <%s> tracker:unmountDate ?unknown } WHERE { <%s> a tracker:Volume; tracker:unmountDate ?unknown } ",
+		                        "DELETE FROM <%s> { <%s> tracker:unmountDate ?unknown } WHERE { <%s> a tracker:Volume; tracker:unmountDate ?unknown } ",
+		                        removable_device_urn, removable_device_urn, removable_device_urn);
+
+		g_string_append_printf (queries,
+		                        "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:unmountDate \"%s\" } ",
+		                        removable_device_urn, removable_device_urn, now);
+
+		g_string_append_printf (queries,
+		                        "DELETE FROM <%s> { <%s> tracker:isMounted ?unknown } WHERE { <%s> a tracker:Volume; tracker:isMounted ?unknown } ",
+		                        removable_device_urn, removable_device_urn, removable_device_urn);
+
+		g_string_append_printf (queries,
+		                        "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:isMounted false } ",
 		                        removable_device_urn, removable_device_urn);
 
 		g_string_append_printf (queries,
-		                        "INSERT { <%s> a tracker:Volume; tracker:unmountDate \"%s\" } ",
-		                        removable_device_urn, now);
-
-		g_string_append_printf (queries,
-		                        "DELETE { <%s> tracker:isMounted ?unknown } WHERE { <%s> a tracker:Volume; tracker:isMounted ?unknown } ",
+		                        "DELETE FROM <%s> { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> } ",
 		                        removable_device_urn, removable_device_urn);
-
-		g_string_append_printf (queries,
-		                        "INSERT { <%s> a tracker:Volume; tracker:isMounted false } ",
-		                        removable_device_urn);
-
-		g_string_append_printf (queries,
-		                        "DELETE { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> } ",
-		                        removable_device_urn);
 		g_free (now);
 	}
 
 	if (accumulator) {
 		g_string_append_printf (accumulator, "%s ", queries->str);
 	} else {
-
-		tracker_miner_execute_update (TRACKER_MINER (miner), queries->str, &error);
-
-		if (error) {
-			g_critical ("Could not set up mount point '%s': %s",
-			            removable_device_urn, error->message);
-			g_error_free (error);
-		}
+		tracker_miner_execute_update (TRACKER_MINER (miner),
+					      queries->str,
+					      NULL,
+					      set_up_mount_point_cb,
+					      g_strdup (removable_device_urn));
 	}
 
 	g_string_free (queries, TRUE);
 }
 
 static void
-init_mount_points (TrackerMinerFiles *miner)
+init_mount_points_cb (GObject      *source,
+                      GAsyncResult *result,
+                      gpointer      user_data)
 {
+	GError *error = NULL;
+	tracker_miner_execute_update_finish (TRACKER_MINER (source),
+	                                     result,
+	                                     &error);
+
+	if (error) {
+		g_critical ("Could not initialize currently active mount points: %s",
+			    error->message);
+	}
+}
+
+static void
+query_mount_points_cb (GObject      *source,
+                       GAsyncResult *result,
+                       gpointer      user_data)
+{
+	TrackerMiner *miner = TRACKER_MINER (source);
 	TrackerMinerFilesPrivate *priv;
 	GHashTable *volumes;
 	GHashTableIter iter;
 	gpointer key, value;
-	GPtrArray *sparql_result;
-	GError *error = NULL;
 	GString *accumulator;
 	gint i;
+	GError *error = NULL;
+	GPtrArray *query_results;
 #ifdef HAVE_HAL
 	GSList *udis, *u;
 #endif
 
-	priv = TRACKER_MINER_FILES_GET_PRIVATE (miner);
+	query_results = tracker_miner_execute_sparql_finish (miner,
+	                                                     result,
+	                                                     &error);
+	if (error) {
+		g_critical ("Could not obtain the mounted volumes: %s", error->message);
+		return;
+	}
 
-	g_debug ("Initializing mount points");
+	priv = TRACKER_MINER_FILES_GET_PRIVATE (miner);
 
 	volumes = g_hash_table_new_full (g_str_hash, g_str_equal,
 	                                 (GDestroyNotify) g_free,
 	                                 NULL);
 
-	/* First, get all mounted volumes, according to tracker-store */
-	sparql_result = tracker_miner_execute_sparql (TRACKER_MINER (miner),
-	                                              "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
-	                                              &error);
-
-	if (error) {
-		g_critical ("Could not obtain the mounted volumes");
-		g_error_free (error);
-		return;
-	}
-
-	for (i = 0; i < sparql_result->len; i++) {
+	for (i = 0; i < query_results->len; i++) {
 		gchar **row;
 		gint state;
 
-		row = g_ptr_array_index (sparql_result, i);
+		row = g_ptr_array_index (query_results, i);
 		state = VOLUME_MOUNTED_IN_STORE;
 
 		if (strcmp (row[0], TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN) == 0) {
@@ -562,9 +598,6 @@ init_mount_points (TrackerMinerFiles *miner)
 		g_hash_table_insert (volumes, g_strdup (row[0]), GINT_TO_POINTER (state));
 	}
 
-	g_ptr_array_foreach (sparql_result, (GFunc) g_strfreev, NULL);
-	g_ptr_array_free (sparql_result, TRUE);
-
 	g_hash_table_replace (volumes, g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN),
 	                      GINT_TO_POINTER (VOLUME_MOUNTED));
 
@@ -611,30 +644,49 @@ init_mount_points (TrackerMinerFiles *miner)
 			}
 #endif
 
-			g_debug ("URN '%s' (mount point: %s) was not reported to be mounted, but now it is, updating state",
-			         mount_point, urn);
-			set_up_mount_point (miner, urn, mount_point, TRUE, accumulator);
+			if (urn) {
+				if (mount_point) {
+					g_debug ("URN '%s' (mount point: %s) was not reported to be mounted, but now it is, updating state",
+						 urn, mount_point);
+				} else {
+					g_debug ("URN '%s' was not reported to be mounted, but now it is, updating state", urn);
+				}
+				set_up_mount_point (TRACKER_MINER_FILES (miner), urn, mount_point, TRUE, accumulator);
+			}
 		} else if (!(state & VOLUME_MOUNTED) &&
 			   (state & VOLUME_MOUNTED_IN_STORE)) {
-			g_debug ("URN '%s' was reported to be mounted, but it isn't anymore, updating state", urn);
-			set_up_mount_point (miner, urn, NULL, FALSE, accumulator);
+			if (urn) {
+				g_debug ("URN '%s' was reported to be mounted, but it isn't anymore, updating state", urn);
+				set_up_mount_point (TRACKER_MINER_FILES (miner), urn, NULL, FALSE, accumulator);
+			}
 		}
 	}
 
 	if (accumulator->str[0] != '\0') {
-		tracker_miner_execute_update (TRACKER_MINER (miner), accumulator->str, &error);
-
-		if (error) {
-			g_critical ("Could not initialize currently active mount points: %s",
-			            error->message);
-			g_error_free (error);
-		}
+		tracker_miner_execute_update (miner,
+					      accumulator->str,
+					      NULL,
+					      init_mount_points_cb,
+					      NULL);
 	}
 
 	g_string_free (accumulator, TRUE);
 	g_hash_table_unref (volumes);
 }
 
+static void
+init_mount_points (TrackerMinerFiles *miner)
+{
+	g_debug ("Initializing mount points");
+
+	/* First, get all mounted volumes, according to tracker-store */
+	tracker_miner_execute_sparql (TRACKER_MINER (miner),
+				      "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
+				      NULL,
+				      query_mount_points_cb,
+				      NULL);
+}
+
 #ifdef HAVE_HAL
 
 static void
@@ -1186,7 +1238,6 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 	}
 
 	if (sparql) {
-		tracker_sparql_builder_insert_close (data->sparql);
 		tracker_sparql_builder_append (data->sparql, sparql);
 		g_free (sparql);
 	}
@@ -1256,7 +1307,7 @@ process_file_cb (GObject      *object,
 	uri = g_file_get_uri (file);
 	mime_type = g_file_info_get_content_type (file_info);
 
-        tracker_sparql_builder_insert_open (sparql);
+        tracker_sparql_builder_insert_open (sparql, uri);
 
         tracker_sparql_builder_subject_iri (sparql, uri);
 	tracker_sparql_builder_predicate (sparql, "a");
@@ -1301,6 +1352,8 @@ process_file_cb (GObject      *object,
 
         miner_files_add_to_datasource (data->miner, file, sparql);
 
+	tracker_sparql_builder_insert_close (sparql);
+
 	/* Send file/mime data to thumbnailer (which adds it to the
 	 * queue if the thumbnailer handles those mime types).
 	 */
@@ -1346,6 +1399,71 @@ miner_files_process_file (TrackerMinerFS       *fs,
 	return TRUE;
 }
 
+static gboolean
+miner_files_writeback_file (TrackerMinerFS       *fs,
+                            GFile                *file,
+                            TrackerSparqlBuilder *sparql,
+                            GCancellable         *cancellable)
+{
+	const gchar *attrs;
+	const gchar *mime_type;
+	GFileInfo *file_info;
+	guint64 time_;
+	gchar *uri;
+	GError *error = NULL;
+
+	attrs = G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+		G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+		G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
+		G_FILE_ATTRIBUTE_STANDARD_SIZE ","
+		G_FILE_ATTRIBUTE_TIME_MODIFIED ","
+		G_FILE_ATTRIBUTE_TIME_ACCESS;
+
+	file_info = g_file_query_info (file, attrs,
+	                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+	                               cancellable, &error);
+
+	if (error) {
+		g_warning ("Can't writeback: '%s'", error->message);
+		g_clear_error (&error);
+		return FALSE;
+	}
+
+	uri = g_file_get_uri (file);
+	mime_type = g_file_info_get_content_type (file_info);
+
+	/* For writeback we only write a few properties back. These properties
+	 * should NEVER be marked as tracker:writeback in the ontology ! */
+
+	tracker_sparql_builder_insert_open (sparql, uri);
+
+	tracker_sparql_builder_subject_iri (sparql, uri);
+	tracker_sparql_builder_predicate (sparql, "a");
+	tracker_sparql_builder_object (sparql, "nfo:FileDataObject");
+
+	tracker_sparql_builder_predicate (sparql, "nfo:fileSize");
+	tracker_sparql_builder_object_int64 (sparql, g_file_info_get_size (file_info));
+
+	time_ = g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
+	tracker_sparql_builder_predicate (sparql, "nfo:fileLastModified");
+	tracker_sparql_builder_object_date (sparql, (time_t *) &time_);
+
+	time_ = g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_ACCESS);
+	tracker_sparql_builder_predicate (sparql, "nfo:fileLastAccessed");
+	tracker_sparql_builder_object_date (sparql, (time_t *) &time_);
+
+	tracker_sparql_builder_predicate (sparql, "nie:mimeType");
+	tracker_sparql_builder_object_string (sparql, mime_type);
+
+	g_object_unref (file_info);
+	g_free (uri);
+
+	tracker_sparql_builder_insert_close (sparql);
+
+	return TRUE;
+}
+
+
 TrackerMiner *
 tracker_miner_files_new (TrackerConfig *config)
 {
