@@ -75,7 +75,7 @@ GType tracker_utils_get_type (void);
 enum  {
 	TRACKER_UTILS_DUMMY_PROPERTY
 };
-char* tracker_utils_EscapeSparql (const char* sparql);
+char* tracker_utils_EscapeSparql (const char* sparql, gboolean add_wildcard);
 gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir);
 GdkPixbuf* tracker_utils_GetThemeIconPixbuf (GIcon* icon, gint size, GdkScreen* screen);
 GdkPixbuf* tracker_utils_GetThemePixbufByName (const char* icon_name, gint size, GdkScreen* screen);
@@ -107,7 +107,7 @@ static glong string_get_length (const char* self) {
 
 
 #line 54 "tracker-utils.gs"
-char* tracker_utils_EscapeSparql (const char* sparql) {
+char* tracker_utils_EscapeSparql (const char* sparql, gboolean add_wildcard) {
 #line 112 "tracker-utils.c"
 	char* result;
 	GString* str;
@@ -162,24 +162,35 @@ char* tracker_utils_EscapeSparql (const char* sparql) {
 		p++;
 #line 164 "tracker-utils.c"
 	}
+#line 76 "tracker-utils.gs"
+	if (add_wildcard) {
+#line 77 "tracker-utils.gs"
+		p--;
+#line 78 "tracker-utils.gs"
+		if ((*p) != ' ') {
+#line 79 "tracker-utils.gs"
+			g_string_append_c (str, '*');
+#line 174 "tracker-utils.c"
+		}
+	}
 	result = g_strdup (str->str);
 	_g_string_free0 (str);
 	return result;
 }
 
 
-#line 81 "tracker-utils.gs"
+#line 87 "tracker-utils.gs"
 gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
-#line 174 "tracker-utils.c"
+#line 185 "tracker-utils.c"
 	gboolean result;
 	GError * _inner_error_;
 	char* command;
 	GAppInfo* app_info;
 	GFile* file;
 	gboolean _tmp2_;
-#line 81 "tracker-utils.gs"
+#line 87 "tracker-utils.gs"
 	g_return_val_if_fail (uri != NULL, FALSE);
-#line 183 "tracker-utils.c"
+#line 194 "tracker-utils.c"
 	_inner_error_ = NULL;
 	command = NULL;
 	app_info = NULL;
@@ -189,15 +200,15 @@ gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
 		GAppInfo* _tmp1_;
 		_tmp0_ = g_file_query_default_handler (file, NULL, &_inner_error_);
 		if (_inner_error_ != NULL) {
-			goto __catch6_g_error;
-			goto __finally6;
+			goto __catch9_g_error;
+			goto __finally9;
 		}
-#line 88 "tracker-utils.gs"
+#line 94 "tracker-utils.gs"
 		app_info = (_tmp1_ = _tmp0_, _g_object_unref0 (app_info), _tmp1_);
-#line 198 "tracker-utils.c"
+#line 209 "tracker-utils.c"
 	}
-	goto __finally6;
-	__catch6_g_error:
+	goto __finally9;
+	__catch9_g_error:
 	{
 		GError * e;
 		e = _inner_error_;
@@ -205,9 +216,9 @@ gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
 		{
 			GtkMessageDialog* msg;
 			msg = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, N_ ("Could not get application info for %s\nError: %s\n"), uri, e->message));
-#line 92 "tracker-utils.gs"
+#line 98 "tracker-utils.gs"
 			gtk_dialog_run ((GtkDialog*) msg);
-#line 211 "tracker-utils.c"
+#line 222 "tracker-utils.c"
 			result = FALSE;
 			_g_error_free0 (e);
 			_g_object_unref0 (msg);
@@ -217,7 +228,7 @@ gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
 			return result;
 		}
 	}
-	__finally6:
+	__finally9:
 	if (_inner_error_ != NULL) {
 		_g_free0 (command);
 		_g_object_unref0 (app_info);
@@ -227,42 +238,42 @@ gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
 		return FALSE;
 	}
 	_tmp2_ = FALSE;
-#line 95 "tracker-utils.gs"
+#line 101 "tracker-utils.gs"
 	if (is_dir == TRUE) {
-#line 95 "tracker-utils.gs"
+#line 101 "tracker-utils.gs"
 		_tmp2_ = _vala_strcmp0 (g_app_info_get_executable (app_info), "nautilus") == 0;
-#line 235 "tracker-utils.c"
+#line 246 "tracker-utils.c"
 	} else {
-#line 95 "tracker-utils.gs"
+#line 101 "tracker-utils.gs"
 		_tmp2_ = FALSE;
-#line 239 "tracker-utils.c"
+#line 250 "tracker-utils.c"
 	}
-#line 95 "tracker-utils.gs"
+#line 101 "tracker-utils.gs"
 	if (_tmp2_) {
-#line 243 "tracker-utils.c"
+#line 254 "tracker-utils.c"
 		char* _tmp4_;
 		char* _tmp3_;
-#line 96 "tracker-utils.gs"
+#line 102 "tracker-utils.gs"
 		command = (_tmp4_ = g_strconcat (_tmp3_ = g_strconcat ("nautilus --sm-disable --no-desktop --no-default-window '", uri, NULL), "'", NULL), _g_free0 (command), _tmp4_);
-#line 248 "tracker-utils.c"
+#line 259 "tracker-utils.c"
 		_g_free0 (_tmp3_);
 	} else {
 		char* _tmp7_;
 		char* _tmp6_;
 		char* _tmp5_;
-#line 98 "tracker-utils.gs"
+#line 104 "tracker-utils.gs"
 		command = (_tmp7_ = g_strconcat (_tmp6_ = g_strconcat (_tmp5_ = g_strconcat (g_app_info_get_executable (app_info), " '", NULL), uri, NULL), "'", NULL), _g_free0 (command), _tmp7_);
-#line 256 "tracker-utils.c"
+#line 267 "tracker-utils.c"
 		_g_free0 (_tmp6_);
 		_g_free0 (_tmp5_);
 	}
 	{
-#line 101 "tracker-utils.gs"
+#line 107 "tracker-utils.gs"
 		g_spawn_command_line_async (command, &_inner_error_);
-#line 263 "tracker-utils.c"
+#line 274 "tracker-utils.c"
 		if (_inner_error_ != NULL) {
-			goto __catch7_g_error;
-			goto __finally7;
+			goto __catch10_g_error;
+			goto __finally10;
 		}
 		result = TRUE;
 		_g_free0 (command);
@@ -270,8 +281,8 @@ gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
 		_g_object_unref0 (file);
 		return result;
 	}
-	goto __finally7;
-	__catch7_g_error:
+	goto __finally10;
+	__catch10_g_error:
 	{
 		GError * e;
 		e = _inner_error_;
@@ -279,9 +290,9 @@ gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
 		{
 			GtkMessageDialog* msg;
 			msg = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, N_ ("Could not lauch %s\nError: %s\n"), uri, e->message));
-#line 106 "tracker-utils.gs"
+#line 112 "tracker-utils.gs"
 			gtk_dialog_run ((GtkDialog*) msg);
-#line 285 "tracker-utils.c"
+#line 296 "tracker-utils.c"
 			result = FALSE;
 			_g_error_free0 (e);
 			_g_object_unref0 (msg);
@@ -291,7 +302,7 @@ gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
 			return result;
 		}
 	}
-	__finally7:
+	__finally10:
 	if (_inner_error_ != NULL) {
 		_g_free0 (command);
 		_g_object_unref0 (app_info);
@@ -306,16 +317,16 @@ gboolean tracker_utils_OpenUri (const char* uri, gboolean is_dir) {
 }
 
 
-#line 110 "tracker-utils.gs"
+#line 116 "tracker-utils.gs"
 inline GdkPixbuf* tracker_utils_GetThemePixbufByName (const char* icon_name, gint size, GdkScreen* screen) {
-#line 312 "tracker-utils.c"
+#line 323 "tracker-utils.c"
 	GdkPixbuf* result;
 	GThemedIcon* icon;
-#line 110 "tracker-utils.gs"
+#line 116 "tracker-utils.gs"
 	g_return_val_if_fail (icon_name != NULL, NULL);
-#line 110 "tracker-utils.gs"
+#line 116 "tracker-utils.gs"
 	g_return_val_if_fail (screen != NULL, NULL);
-#line 319 "tracker-utils.c"
+#line 330 "tracker-utils.c"
 	icon = (GThemedIcon*) g_themed_icon_new (icon_name);
 	result = tracker_utils_GetThemeIconPixbuf ((GIcon*) icon, size, screen);
 	_g_object_unref0 (icon);
@@ -323,56 +334,74 @@ inline GdkPixbuf* tracker_utils_GetThemePixbufByName (const char* icon_name, gin
 }
 
 
-#line 117 "tracker-utils.gs"
+#line 123 "tracker-utils.gs"
 GdkPixbuf* tracker_utils_GetThumbNail (GFileInfo* info, gint thumb_size, gint icon_size, GdkScreen* screen) {
-#line 329 "tracker-utils.c"
+#line 340 "tracker-utils.c"
 	GdkPixbuf* result;
 	GError * _inner_error_;
 	GdkPixbuf* pixbuf;
-	char* thumbpath;
-#line 117 "tracker-utils.gs"
+#line 123 "tracker-utils.gs"
 	g_return_val_if_fail (info != NULL, NULL);
-#line 117 "tracker-utils.gs"
+#line 123 "tracker-utils.gs"
 	g_return_val_if_fail (screen != NULL, NULL);
-#line 338 "tracker-utils.c"
+#line 348 "tracker-utils.c"
 	_inner_error_ = NULL;
 	pixbuf = NULL;
-	thumbpath = g_strdup (g_file_info_get_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH));
-#line 123 "tracker-utils.gs"
-	if (thumbpath != NULL) {
-#line 344 "tracker-utils.c"
-		GdkPixbuf* _tmp0_;
-		GdkPixbuf* _tmp1_;
-		_tmp0_ = gdk_pixbuf_new_from_file_at_size (thumbpath, thumb_size, thumb_size, &_inner_error_);
-		if (_inner_error_ != NULL) {
-			_g_object_unref0 (pixbuf);
-			_g_free0 (thumbpath);
-			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
-			g_clear_error (&_inner_error_);
-			return NULL;
-		}
-#line 124 "tracker-utils.gs"
-		pixbuf = (_tmp1_ = _tmp0_, _g_object_unref0 (pixbuf), _tmp1_);
-#line 357 "tracker-utils.c"
-	}
-#line 126 "tracker-utils.gs"
-	if (pixbuf == NULL) {
-#line 361 "tracker-utils.c"
-		GdkPixbuf* _tmp2_;
-#line 127 "tracker-utils.gs"
-		pixbuf = (_tmp2_ = tracker_utils_GetThemeIconPixbuf (g_file_info_get_icon (info), icon_size, screen), _g_object_unref0 (pixbuf), _tmp2_);
-#line 365 "tracker-utils.c"
-	}
-#line 129 "tracker-utils.gs"
-	if (pixbuf == NULL) {
-#line 369 "tracker-utils.c"
-		GdkPixbuf* _tmp3_;
+	{
+		char* thumbpath;
+		thumbpath = g_strdup (g_file_info_get_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH));
 #line 130 "tracker-utils.gs"
+		if (thumbpath != NULL) {
+#line 356 "tracker-utils.c"
+			GdkPixbuf* _tmp0_;
+			GdkPixbuf* _tmp1_;
+			_tmp0_ = gdk_pixbuf_new_from_file_at_size (thumbpath, thumb_size, thumb_size, &_inner_error_);
+			if (_inner_error_ != NULL) {
+				_g_free0 (thumbpath);
+				goto __catch11_g_error;
+				goto __finally11;
+			}
+#line 131 "tracker-utils.gs"
+			pixbuf = (_tmp1_ = _tmp0_, _g_object_unref0 (pixbuf), _tmp1_);
+#line 367 "tracker-utils.c"
+		}
+		_g_free0 (thumbpath);
+	}
+	goto __finally11;
+	__catch11_g_error:
+	{
+		GError * e;
+		e = _inner_error_;
+		_inner_error_ = NULL;
+		{
+			;
+			_g_error_free0 (e);
+		}
+	}
+	__finally11:
+	if (_inner_error_ != NULL) {
+		_g_object_unref0 (pixbuf);
+		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
+		g_clear_error (&_inner_error_);
+		return NULL;
+	}
+#line 135 "tracker-utils.gs"
+	if (pixbuf == NULL) {
+#line 391 "tracker-utils.c"
+		GdkPixbuf* _tmp2_;
+#line 136 "tracker-utils.gs"
+		pixbuf = (_tmp2_ = tracker_utils_GetThemeIconPixbuf (g_file_info_get_icon (info), icon_size, screen), _g_object_unref0 (pixbuf), _tmp2_);
+#line 395 "tracker-utils.c"
+	}
+#line 138 "tracker-utils.gs"
+	if (pixbuf == NULL) {
+#line 399 "tracker-utils.c"
+		GdkPixbuf* _tmp3_;
+#line 139 "tracker-utils.gs"
 		pixbuf = (_tmp3_ = tracker_utils_GetThemePixbufByName ("text-x-generic", icon_size, screen), _g_object_unref0 (pixbuf), _tmp3_);
-#line 373 "tracker-utils.c"
+#line 403 "tracker-utils.c"
 	}
 	result = pixbuf;
-	_g_free0 (thumbpath);
 	return result;
 }
 
@@ -382,27 +411,52 @@ static gpointer _g_object_ref0 (gpointer self) {
 }
 
 
-#line 135 "tracker-utils.gs"
+#line 144 "tracker-utils.gs"
 GdkPixbuf* tracker_utils_GetThemeIconPixbuf (GIcon* icon, gint size, GdkScreen* screen) {
-#line 388 "tracker-utils.c"
+#line 417 "tracker-utils.c"
 	GdkPixbuf* result;
 	GError * _inner_error_;
 	GtkIconInfo* icon_info;
 	GtkIconTheme* theme;
 	GtkIconInfo* _tmp0_;
-	GdkPixbuf* _tmp1_;
-#line 135 "tracker-utils.gs"
+#line 144 "tracker-utils.gs"
 	g_return_val_if_fail (icon != NULL, NULL);
-#line 135 "tracker-utils.gs"
+#line 144 "tracker-utils.gs"
 	g_return_val_if_fail (screen != NULL, NULL);
-#line 399 "tracker-utils.c"
+#line 427 "tracker-utils.c"
 	_inner_error_ = NULL;
 	icon_info = NULL;
 	theme = _g_object_ref0 (gtk_icon_theme_get_for_screen (screen));
-#line 141 "tracker-utils.gs"
+#line 150 "tracker-utils.gs"
 	icon_info = (_tmp0_ = gtk_icon_theme_lookup_by_gicon (theme, icon, size, GTK_ICON_LOOKUP_USE_BUILTIN), _gtk_icon_info_free0 (icon_info), _tmp0_);
-#line 405 "tracker-utils.c"
-	_tmp1_ = gtk_icon_info_load_icon (icon_info, &_inner_error_);
+#line 433 "tracker-utils.c"
+	{
+		GdkPixbuf* _tmp1_;
+		_tmp1_ = gtk_icon_info_load_icon (icon_info, &_inner_error_);
+		if (_inner_error_ != NULL) {
+			goto __catch12_g_error;
+			goto __finally12;
+		}
+		result = _g_object_ref0 (_tmp1_);
+		_gtk_icon_info_free0 (icon_info);
+		_g_object_unref0 (theme);
+		return result;
+	}
+	goto __finally12;
+	__catch12_g_error:
+	{
+		GError * e;
+		e = _inner_error_;
+		_inner_error_ = NULL;
+		{
+			result = NULL;
+			_g_error_free0 (e);
+			_gtk_icon_info_free0 (icon_info);
+			_g_object_unref0 (theme);
+			return result;
+		}
+	}
+	__finally12:
 	if (_inner_error_ != NULL) {
 		_gtk_icon_info_free0 (icon_info);
 		_g_object_unref0 (theme);
@@ -410,16 +464,14 @@ GdkPixbuf* tracker_utils_GetThemeIconPixbuf (GIcon* icon, gint size, GdkScreen*
 		g_clear_error (&_inner_error_);
 		return NULL;
 	}
-	result = _g_object_ref0 (_tmp1_);
 	_gtk_icon_info_free0 (icon_info);
 	_g_object_unref0 (theme);
-	return result;
 }
 
 
 #line 51 "tracker-utils.gs"
 TrackerUtils* tracker_utils_construct (GType object_type) {
-#line 423 "tracker-utils.c"
+#line 475 "tracker-utils.c"
 	TrackerUtils* self;
 	self = (TrackerUtils*) g_type_create_instance (object_type);
 	return self;
@@ -430,7 +482,7 @@ TrackerUtils* tracker_utils_construct (GType object_type) {
 TrackerUtils* tracker_utils_new (void) {
 #line 51 "tracker-utils.gs"
 	return tracker_utils_construct (TYPE_TRACKER_UTILS);
-#line 434 "tracker-utils.c"
+#line 486 "tracker-utils.c"
 }
 
 
