@@ -41,6 +41,7 @@
 #include "tracker-resources.h"
 #include "tracker-resource-class.h"
 #include "tracker-events.h"
+#include "tracker-writeback.h"
 #include "tracker-store.h"
 
 #define RDF_PREFIX TRACKER_RDF_PREFIX
@@ -51,9 +52,13 @@ G_DEFINE_TYPE(TrackerResources, tracker_resources, G_TYPE_OBJECT)
 
 #define TRACKER_RESOURCES_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_RESOURCES, TrackerResourcesPrivate))
 
+enum {
+	WRITEBACK,
+	LAST_SIGNAL
+};
 
 typedef struct {
-	GSList     *event_sources;
+	GSList *event_sources;
 } TrackerResourcesPrivate;
 
 typedef struct {
@@ -61,6 +66,9 @@ typedef struct {
 	guint request_id;
 } TrackerDBusMethodInfo;
 
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void
 free_event_sources (TrackerResourcesPrivate *priv)
 {
@@ -94,6 +102,16 @@ tracker_resources_class_init (TrackerResourcesClass *klass)
 
 	object_class->finalize = tracker_resources_finalize;
 
+	signals[WRITEBACK] =
+		g_signal_new ("writeback",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (TrackerResourcesClass, writeback),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__BOXED,
+			      G_TYPE_NONE, 1,
+			      TRACKER_TYPE_STR_STRV_MAP);
+
 	g_type_class_add_private (object_class, sizeof (TrackerResourcesPrivate));
 }
 
@@ -250,6 +268,44 @@ tracker_resources_sparql_update (TrackerResources	 *self,
 	tracker_dbus_request_success (request_id);
 }
 
+void
+tracker_resources_sparql_update_blank (TrackerResources	      *self,
+				       const gchar	      *update,
+				       DBusGMethodInvocation  *context,
+				       GError		     **error)
+{
+	TrackerResourcesPrivate *priv;
+	GError 		     *actual_error = NULL;
+	guint		      request_id;
+	GPtrArray            *blank_nodes;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
+
+	request_id = tracker_dbus_get_next_request_id ();
+
+	tracker_dbus_async_return_if_fail (update != NULL, context);
+
+	tracker_dbus_request_new (request_id,
+				  "D-Bus request for SPARQL Update, "
+				  "update:'%s'",
+				  update);
+
+	blank_nodes = tracker_store_sparql_update_blank (update, &actual_error);
+
+	if (actual_error) {
+		tracker_dbus_request_failed (request_id,
+					     &actual_error,
+					     NULL);
+		dbus_g_method_return_error (context, actual_error);
+		g_error_free (actual_error);
+		return;
+	}
+
+	dbus_g_method_return (context, blank_nodes);
+
+	tracker_dbus_request_success (request_id);
+}
+
 static void
 batch_update_callback (GError *error, gpointer user_data)
 {
@@ -346,15 +402,18 @@ tracker_resources_batch_commit (TrackerResources	 *self,
 static void
 on_statements_committed (gpointer user_data)
 {
+	TrackerResources *resources = user_data;
 	GPtrArray *events;
+	GHashTable *writebacks;
 	TrackerResourcesPrivate *priv;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (user_data);
+	priv = TRACKER_RESOURCES_GET_PRIVATE (resources);
 
 	/* For more information about this call, look at the function end_batch
 	 * of tracker-store.c */
 	tracker_store_flush_journal ();
 
+	/* Class signals feature */
 	events = tracker_events_get_pending ();
 
 	if (events) {
@@ -362,7 +421,7 @@ on_statements_committed (gpointer user_data)
 		guint i;
 		GHashTable *to_emit = NULL;
 
-		event_sources =priv->event_sources;
+		event_sources = priv->event_sources;
 
 		for (i = 0; i < events->len; i++) {
 			GValueArray *event = events->pdata[i];
@@ -399,6 +458,16 @@ on_statements_committed (gpointer user_data)
 	}
 
 	tracker_events_reset ();
+
+	/* Writeback feature */
+	writebacks = tracker_writeback_get_pending ();
+
+	if (writebacks) {
+		g_signal_emit (resources, signals[WRITEBACK], 0, writebacks);
+
+	}
+
+	tracker_writeback_reset ();
 }
 
 
@@ -406,6 +475,7 @@ static void
 on_statements_rolled_back (gpointer user_data)
 {
 	tracker_events_reset ();
+	tracker_writeback_reset ();
 }
 
 static void
@@ -421,6 +491,9 @@ on_statement_inserted (const gchar *graph,
 	} else {
 		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_UPDATE);
 	}
+
+	/* For predicates it's always update here */
+	tracker_writeback_check (graph, subject, predicate, object, rdf_types);
 }
 
 static void
@@ -436,8 +509,10 @@ on_statement_deleted (const gchar *graph,
 	} else {
 		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_UPDATE);
 	}
-}
 
+	/* For predicates it's always delete here */
+	tracker_writeback_check (graph, subject, predicate, object, rdf_types);
+}
 
 void 
 tracker_resources_prepare (TrackerResources *object,
@@ -463,3 +538,4 @@ tracker_resources_unreg_batches (TrackerResources *object,
 {
 	tracker_store_unreg_batches (old_owner);
 }
+
