@@ -31,6 +31,16 @@
 #include <libtracker-client/tracker.h>
 #include <libtracker-common/tracker-common.h>
 
+#define ABOUT                                                             \
+        "Tracker " PACKAGE_VERSION "\n"
+
+#define LICENSE                                                           \
+        "This program is free software and comes without any warranty.\n" \
+        "It is licensed under version 2 or later of the General Public "  \
+        "License which can be viewed at:\n"                               \
+        "\n"                                                              \
+        "  http://www.gnu.org/licenses/gpl.txt\n"
+
 static gint	      limit = 512;
 static gint	      offset;
 static gchar	    **terms;
@@ -44,6 +54,7 @@ static gboolean       music_files;
 static gboolean       image_files;
 static gboolean       video_files;
 static gboolean       document_files;
+static gboolean       print_version;
 
 static GOptionEntry   entries[] = {
 	{ "limit", 'l', 0, G_OPTION_ARG_INT, &limit,
@@ -94,6 +105,10 @@ static GOptionEntry   entries[] = {
 	  N_("Search for document files"),
 	  NULL
 	},
+	{ "version", 'V', 0, G_OPTION_ARG_NONE, &print_version,
+	  N_("Print version"),
+	  NULL
+	},
 	{ G_OPTION_REMAINING, 0, 0,
 	  G_OPTION_ARG_STRING_ARRAY, &terms,
 	  N_("search terms"),
@@ -793,7 +808,14 @@ main (int argc, char **argv)
 	g_option_context_parse (context, &argc, &argv, NULL);
 
 	g_free (summary);
-	
+
+	if (print_version) {
+		g_print ("\n" ABOUT "\n" LICENSE "\n");
+		g_option_context_free (context);
+
+		return EXIT_SUCCESS;
+	}
+
 	if (!music_albums && !music_artists && !music_files &&
 	    !image_files &&
 	    !video_files &&
