@@ -31,6 +31,16 @@
 #include <libtracker-client/tracker.h>
 #include <libtracker-common/tracker-common.h>
 
+#define ABOUT                                                             \
+        "Tracker " PACKAGE_VERSION "\n"
+
+#define LICENSE                                                           \
+        "This program is free software and comes without any warranty.\n" \
+        "It is licensed under version 2 or later of the General Public "  \
+        "License which can be viewed at:\n"                               \
+        "\n"                                                              \
+        "  http://www.gnu.org/licenses/gpl.txt\n"
+
 static gint	     limit = 512;
 static gint	     offset;
 static gchar	   **files;
@@ -39,6 +49,7 @@ static gchar	    *add_tag;
 static gchar	    *remove_tag;
 static gboolean     *list;
 static gboolean      show_files;
+static gboolean      print_version;
 
 static GOptionEntry entries[] = {
 	{ "limit", 'l', 0, G_OPTION_ARG_INT, &limit,
@@ -69,6 +80,10 @@ static GOptionEntry entries[] = {
 	  N_("Delete a tag (if FILEs are omitted, TAG is removed for all files)"),
 	  N_("TAG")
 	},
+	{ "version", 'V', 0, G_OPTION_ARG_NONE, &print_version,
+	  N_("Print version"),
+	  NULL
+	},
 	{ G_OPTION_REMAINING, 0, 
 	  G_OPTION_FLAG_FILENAME,
 	  G_OPTION_ARG_STRING_ARRAY, &files,
@@ -629,6 +644,13 @@ main (int argc, char **argv)
 	g_option_context_add_main_entries (context, entries, NULL);
 	g_option_context_parse (context, &argc, &argv, NULL);
 
+	if (print_version) {
+                g_print ("\n" ABOUT "\n" LICENSE "\n");
+		g_option_context_free (context);
+
+		return EXIT_SUCCESS;
+	}
+
 	if (!list && show_files) {
 		failed = _("The --list option is required for --show-files");
 	} else if (add_tag && remove_tag) {
